;;;; vc.vc-base -- basic vc methods

(define-module vc.vc-base
  (use srfi-2)  ;; and-let*
  (use srfi-11) ;; let-values
  (use rfc.uri)
  (export-all))
(select-module vc.vc-base)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; classes

;; abstract version control object
(define-class <vc-object> ()
  ())

;; local or remote repository
(define-class <vc-repository> (<vc-object>)
  ((host :init-keyword :host :init-value #f :getter host-of)
   (port :init-keyword :port :init-value #f :getter port-of)
   (user :init-keyword :user :init-value #f :getter user-of)
   (password :init-keyword :password :init-value #f :getter password-of)
   (path :init-keyword :path :init-value #f :getter path-of)))

;; working sandbox
(define-class <vc-module> (<vc-object>)
  ((repository :init-keyword :repository :init-value #f :getter repository-of)
   (name       :init-keyword :name       :init-value #f :getter name-of)
   ))

;; local or remote directory
(define-class <vc-data> (<vc-object>)
  ((module  :init-keyword :module  :init-value #f :getter module-of)
   (path    :init-keyword :path    :init-value #f :getter path-of)
   (version :init-keyword :version :init-value #f :getter version-of)))

;; local or remote directory
(define-class <vc-dir> (<vc-data>) ())

;; local or remote file
(define-class <vc-file> (<vc-data>) ())

;; a single modification as returned by vc-history and vc-log
;; implementations may not necessarily return all info
(define-class <vc-change> ()
  ((type :init-keyword :type :init-value 'modification :getter type-of)
   (file :init-keyword :file :init-value #f :getter file-of)
   (time :init-keyword :time :init-value #f :getter time-of)
   (author :init-keyword :author :init-value #f :getter author-of)
   (version :init-keyword :version :init-value #f :getter version-of)
   (comment :init-keyword :comment :init-value #f :getter comment-of)))

(define-method repository-of ((obj <vc-repository>))
  obj)

(define-method repository-of ((obj <vc-data>))
  (and-let* ((module (module-of obj)))
    (repository-of module)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; generics

(define-generic vc-new-module)
(define-generic vc-checkout)
(define-generic vc-add)
(define-generic vc-update)
(define-generic vc-edit)
(define-generic vc-commit)
(define-generic vc-remove)
(define-generic vc-revert)
(define-generic vc-rename)

(define-generic vc-tag)
(define-generic vc-log)
(define-generic vc-diff)
(define-generic vc-history)
(define-generic vc-versions)

(define-generic vc-open-input)
(define-generic vc-fetch)

(provide "vc/vc-base")

